/*
 * Projet C L1.2 : Puissance 4
 * Auteur : Daniel Borges
 * affichage.c : contient les fonctions d'affichage
 */

#include "puissance_4.h"

/* Affichage des pions et de la barre de status */
void affichage(Donnees *partie)
{
    affichage_pions(partie);
    affichage_status(partie);
    display_window(&partie->ecran);
    return;
}
/* Affiche les pions */
void affichage_pions(Donnees *partie)
{
    int i,j;
    
    // Affichage tous les pions si c'est le premier tour
    // ou seulement le dernier pion jou
    if ( partie->tour == 0 )
    {
	// Fond de la grille
	draw_filled_rectangle(0,0,partie->tcases*partie->lgrille,partie->tcases*partie->hgrille,
	        CFONDGRILLE,&partie->ecran);
	// Pions
	for ( i=0 ; i < partie->hgrille ; i++ )
	    for ( j=0 ; j < partie->lgrille ; j++ )
		affiche_pion(partie,j,i);
    }
    else
	affiche_pion(partie,partie->x,partie->y);

    return;
}

/* Affiche le pion de la case (x,y) de la grille */
void affiche_pion(Donnees *partie, int x, int y)
{
    int ordonnee,abscisse,rayon;

    // Calcul du rayon d'un pion ( 95% de la taille d'une case )
    rayon = (int)((float)(partie->tcases*95/100/2));

    // Abcisse et ordonne du centre du pion  l'cran
    abscisse = partie->tcases*(x+1)-partie->tcases/2;
    ordonnee = partie->tcases*(y+1)-partie->tcases/2;

    switch(partie->grille[y][x])
    {
        case ROUGE:
	    draw_filled_circle(abscisse,ordonnee,rayon,CROUGE,&partie->ecran);
	    break;
	case JAUNE:
	    draw_filled_circle(abscisse,ordonnee,rayon,CJAUNE,&partie->ecran);
	    break;
	case VIDE:
	    draw_filled_circle(abscisse,ordonnee,rayon,CVIDE,&partie->ecran);
    }

    return;
}

/* Affiche la barre de status comprenant :
 * - tour
 * - bouton annuler
 * - bouton recommencer
 * - bouton quitter
 */
void affichage_status(Donnees *partie)
{
    // Fond blanc pour la barre de status
    draw_filled_rectangle(0,partie->tcases*partie->hgrille,partie->tcases*partie->lgrille,
	    HSTATUS,"white",&partie->ecran);
    
    // Tour
    if ( partie->joueur == ROUGE )
	affichage_message(partie,"Au joueur Rouge de jouer");
    else
	affichage_message(partie,"Au joueur Jaune de jouer");

    affichage_bouton(partie,"Q","red",1); // Bouton quitter
    affichage_bouton(partie,"R","yellow",2); // Bouton recommencer
    affichage_bouton(partie,"A","green",3); // Bouton annuler
    
    return;
}

/* Affiche une question dans la barre de status
 * ainsi que deux boutons O (oui) et N (non) pour rpondre */
void affichage_question(Donnees *partie, char *question)
{
    // Affichage de la question
    affichage_message(partie,question);
    
    // Boutons
    affichage_bouton(partie,"O","green",2); // Bouton Oui
    affichage_bouton(partie,"N","red",1); // Bouton Non

    return;
}

/* Affichage un message indiquant les diffrentes options possibles */
void aide(void)
{
    fprintf(stderr,"Option valides :\n\
 -d x : adresse du serveur X\n\
 -h x : hauteur de la grille de jeu\n\
 -l x : largeur de la grille de jeu\n\
 -p x : nombre de pions  aligner pour gagner\n\
 -t x : taille ( en pixels ) des cases de la grille\n\
 -a : cette page d'aide\n");

    return;
}

/* Affiche qui a gagn la partie */
void gagner(Donnees *partie)
{
    int x,y;

    if ( partie->grille[partie->y][partie->x] == ROUGE )
	affichage_message(partie,"Le joueur Rouge  gagn");
    else
	affichage_message(partie,"Le joueur Jaune  gagn");
    
    mouse_wait(&x,&y,&partie->ecran);

    return;
}

/* Affichage un message dans la barre de status */
void affichage_message(Donnees *partie, char *message)
{
    // Fond
    draw_filled_rectangle(0,partie->tcases*partie->hgrille,partie->tcases*partie->lgrille,
	    HSTATUS,"white",&partie->ecran);

    // Message
    draw_string(2,partie->tcases*partie->hgrille+HSTATUS-2,message,POLICE,
	    "black",&partie->ecran);

    // Affichage
    display_area(0,partie->tcases*partie->hgrille,partie->tcases*partie->lgrille,HSTATUS,&partie->ecran);

    return;
}

/* Affiche un bouton  la position demande ( en partant de la fin )
 * dans la barre de status */
void affichage_bouton(Donnees *partie, char *nom, char *couleur, int position)
{
    // Fond
    draw_filled_rectangle(partie->tcases*partie->lgrille-HSTATUS*position,
	    partie->tcases*partie->hgrille,HSTATUS,HSTATUS,couleur,&partie->ecran);
    // Lettre
    draw_string(partie->tcases*partie->lgrille-HSTATUS*position+3,
	    partie->tcases*partie->hgrille+HSTATUS-2,nom,POLICE,"black",&partie->ecran);

    // Affichage
    display_area(partie->tcases*partie->lgrille-HSTATUS*position,
	    partie->tcases*partie->hgrille,HSTATUS,HSTATUS,&partie->ecran);
    
    return;
}
