/*
 * Projet C L1.2 : Puissance 4
 * Auteur : Daniel Borges
 * es.c : contient les fonctions d'entre/sortie
 */

#include "puissance_4.h"

/* Fonction pour entrer une action
 * - Retourne -1 si on clique sur la case pour quitter le jeu 
 * - Retourne -2 si on clique sur la case pour recommencer la partie
 * - Retourne -3 si on clique sur la case pour annuler le dernier coup 
 * - Retourne un nombre suprieur ou gal  zro correspondant  la colonne
 *   souhaite dans les cas restants
 */
int entree(Donnees *partie)
{
    int choix,x,y;
    
    do {
	mouse_wait(&x,&y,&partie->ecran);

	// Colonne de jeu
	if ( y < partie->tcases*partie->hgrille )
	    choix = x/partie->tcases;
	else
	{
	    // Quitter
	    if ( x > partie->tcases*partie->lgrille-HSTATUS )
		choix = -1;
	    // Recommencer
	    else if ( x > partie->tcases*partie->lgrille-HSTATUS*2 )
		choix = -2;
	    // Annuler le dernier coup
	    else if ( x > partie->tcases*partie->lgrille-HSTATUS*3 )
		choix = -3;
	}
    } while ( x < partie->tcases*partie->lgrille-HSTATUS*3 && y > partie->tcases*partie->hgrille );

    return choix;
}

/* Demande au joueur s'il dsire rejouer, et relance la partie dans ce cas 
 * retourne 1 si la partie  recommenc */
void rejouer(Donnees *partie)
{
    // Affichage de la demande
    affichage_question(partie,"Recommencer ?");

    // Demande de rponse
    if ( reponse(partie) )
    {
	close_window(&partie->ecran);
	init_partie(partie);
    }

    return;
}

/* Demande au joueur s'il dsire rellement quitter et quitte si oui 
 * retourne 0 si on ne quitte pas */
int quitter(Donnees *partie)
{
    // Affichage de la question
    affichage_question(partie,"Quitter ?");

    // Rponse
    if ( reponse(partie) )
    {
	close_window(&partie->ecran);
	exit(0);
    }

    return 0;
}

/* Retourne 1 si la rponse  la question est oui, 0 sinon */
int reponse(Donnees *partie)
{
    int x,y;

    do { mouse_wait(&x,&y,&partie->ecran);
    } while ( x < partie->tcases* partie->lgrille -HSTATUS*2 || y < partie->tcases* partie->hgrille );

    // Non ?
    if ( x > partie->tcases* partie->lgrille -HSTATUS )
	return 0;
    // Oui.
    return 1;    
}

/* Applique les valeurs par dfaut puis les modifie pour correspondre
 * aux options passes sur la ligne de commande */
void options(Donnees *partie, int argc, char **argv)
{
    char c;
    
    // Options par dfaut
    partie->hgrille = HGRILLE;
    partie->lgrille = LGRILLE;
    partie->puissance = PUISSANCE;
    partie->display = DISPLAY;
    partie->tcases = TCASES;

    // Options passes  la ligne de commande
    while ( (c = getopt(argc,argv,"ad:h:l:p:t:")) != -1 )
    {
	switch(c)
	{
	    case 'd': // Adresse du serveur graphique
		partie->display = optarg;
		break;
	    case 'h': // Hauteur de la grille
		partie->hgrille = atoi(optarg);
		break;
	    case 'l': // Largeur de la grille
		partie->lgrille = atoi(optarg);
		break;
	    case 'p': // Puissance
		partie->puissance = atoi(optarg);
		break;
	    case 't': // Taille des cases
		partie->tcases = atoi(optarg);
		break;
	    case 'a': // Aide
		aide();
		exit(0);
	    case '?': // Option inconnue
		aide();
		exit(0);
	}
    }

    // Si la puissance demande est plus grande que jouable dans le tableau
    if ( partie->puissance > partie->hgrille && partie->puissance > partie->lgrille )
    {
	puts("Erreur : puissance trop leve");
	exit(0);
    }

    return;
}

