/*
 * Projet C L1.2 : Puissance 4
 * Auteur : Daniel Borges
 * puissance_4.c : contient les fonctions de jeu
 */

#include "puissance_4.h"

/* Initialisation de la partie */
void init_partie(Donnees *partie)
{
    int i,j;

    /* Initialisations relatives au jeu */
    partie->tour = 0; // Dpart du jeu
    partie->joueur = ROUGE; // Le joueur rouge commence
    // On remplit la grille de "cases vides"
    for ( i=0 ; i < partie->hgrille ; i++ )
    	for ( j=0 ; j < partie->lgrille ; j++ )
	    partie->grille[i][j] = VIDE; 

    /* Initialisations relatives  l'affichage */
    // Cration de la fentre
    create_window(partie->display,partie->tcases*partie->lgrille,
	    partie->tcases*partie->hgrille+HSTATUS,&partie->ecran); 
    // Nom de la fentre
    name_window("Puissance 4",&partie->ecran); 

    return;
}

/* Boucle de jeu : affiche, slectionne, joue */
void jeu(Donnees *partie)
{
    int choix;

    // Temps qu'il y a un joueur et que le tableau n'est pas plein
    do {
	// On affiche
	affichage(partie);

	// On joue
	switch(choix = entree(partie))
	{
	    // Annuler
	    case -3: annuler(partie); break;
	    // Recommencer
	    case -2: rejouer(partie); break;
	    // Quitter
	    case -1: quitter(partie); break;
	    // Jouer
	    default: jouer(partie,choix);
	}
    } while ( partie->joueur && partie->tour < partie->hgrille * partie->lgrille );

    // Pour afficher le dernier pion jou
    affichage(partie);

    // Si il y a un gagnant, on l'affiche
    if ( partie->tour < partie->hgrille * partie->lgrille )
    	gagner(partie);

    // Nouvelle partie
    if (!quitter(partie))
    {
	close_window(&partie->ecran);
	init_partie(partie);
    }

    return;
}

/* Jouer dans la colonne spcifie */
void jouer(Donnees *partie, int colonne)
{
    int ligne;
    
    // On ajoute le pion
    if ( (ligne = ajout_pion(partie,colonne)) != -1 )
    {
	// On note le coup dans l'historique
	partie->historique[partie->tour] = colonne;
	// On passe au tour suivant
	partie->tour++;
	// On note les coordonnes
	partie->x = colonne;
	partie->y = ligne;
	// On passe au joueur suivant s'il n'y a pas de puissance 4
	if ( puissance_4(partie) )
	    partie->joueur = 0;
	else
	    partie->joueur = ( partie->joueur == ROUGE ) ? JAUNE : ROUGE;
    }

    return;
}

/* Annuler le dernier coup */
void annuler(Donnees *partie)
{
    // Si la partie a dj commenc
    if ( partie->tour > 0 ) 
    {
	// On retourne au tour prcdent
	partie->tour--;
	// On remet l'abscisse et l'ordonne du dernier pion jou
	partie->x = partie->historique[partie->tour];
	partie->y = recherche_pion(partie,partie->x);
	// On enlve le pion
	partie->grille[partie->y][partie->x] = VIDE;
	// On retourne au joueur prcdent
	partie->joueur = ( partie->joueur == ROUGE ) ? JAUNE : ROUGE;
    }

    return;
}

/* Retourne le nombre de pions de la couleur du joueur qui joue dans la direction donne.
 * Pour choisir la direction, on utilise un modificateur -1, 0 ou 1 pour l'ordonne 
 * et l'abscisse. Exemple : pour aller vers la gauche, -1 sur x, 0 sur y.
 */
int nombre_pions(Donnees *partie, int dirx, int diry)
{
    int i,j,nombre=0;

    // On dmarre le compte  partie du pion "d' cot"
    for ( i=partie->x+dirx, j=partie->y+diry ;
	    // On reste dans les limites
	    i >= 0 && i < partie->lgrille 
	    && j >= 0 && j < partie->hgrille 
	    // On cherche les pions du joueur
	    && partie->grille[j][i] == partie->joueur ;
	    // On avance dans les directions demandes
	    i += dirx, j += diry )
	nombre++;

    return nombre;
}

/* Retourne 1 si le nombre de pions requis pour gagner est align */
int puissance_4(Donnees *partie)
{
    // Horizontalement
    if ( (nombre_pions(partie,-1,0) + nombre_pions(partie,1,0) + 1) == partie->puissance )
	return 1;
    // Verticalement
    if ( nombre_pions(partie,0,1) + 1 == partie->puissance )
	return 1;
    // Diagonale bas-gauche vers haut-droite
    if ( (nombre_pions(partie,-1,-1) + nombre_pions(partie,1,1) + 1) == partie->puissance )
	return 1;
    // Diagonale bas-droite vers haut-gauche
    if ( (nombre_pions(partie,1,-1) + nombre_pions(partie,-1,1) + 1) == partie->puissance )
	return 1;

    return 0;
}

/* Retourne la ligne sur laquelle 
 * se trouve le plus haut pion de la colonne */
int recherche_pion(Donnees *partie, int colonne)
{
    int ligne=0;

    while ( ligne < partie->hgrille && partie->grille[ligne][colonne] == VIDE )
	ligne++;

    return ligne;
}

/* Ajoute un pion dans la colonne spcifie, 
 * retourne le numro de la ligne sur lequel le pion se trouve s'il a bien t ajout
 * -1 sinon */
int ajout_pion(Donnees *partie, int colonne)
{
    int ligne;

    // Si la colonne n'est pas deja pleine
    if ( (ligne = recherche_pion(partie,colonne)) )
    {
	// On ajoute un pion de la couleur du joueur qui joue
	partie->grille[ligne-1][colonne] = partie->joueur;

	return ligne-1;
    }

    return -1;
}

/* Allocation de mmoire pour la grille et l'historique */
void alloc_mem(Donnees *partie)
{
    int i;
    
    // Mmoire pour la grille
    partie->grille = malloc(sizeof(int *)*partie->hgrille); // lignes
    for ( i=0 ; i < partie->hgrille ; i++ )
	partie->grille[i] = malloc(sizeof(int)*partie->lgrille); // colonnes
    // Mmoire pour l'historique
    partie->historique = malloc(sizeof(int)*partie->hgrille*partie->lgrille);

    return;
}

/* Libre la mmoire utilise par le jeu */
void libere_mem(Donnees *partie)
{
    int i;

    for ( i=0 ; i < partie->hgrille ; i++ )
	free(partie->grille[i]); // colonnes
    free(partie->grille); // lignes
    free(partie->historique); // historique
}

/* initialisation & lancement du jeu */
int main (int argc, char **argv)
{
    Donnees partie;

    // Initialisation
    options(&partie,argc,argv);
    alloc_mem(&partie);
    init_partie(&partie);

    // Lancement du jeu
    while (1) // On assume que les joueurs veulent effectuer plusieurs parties
	jeu(&partie);

    libere_mem(&partie);

    return 0;
}
