/* Inclusions */
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <MlvTypeC.h>
#include <MlvProcC.h>


/* Définitions */
// relatives au jeu
#define LGRILLE 7 // Largeur de la grille par défaut
#define HGRILLE 6 // Hauteur de la grille par défaut
#define PUISSANCE 4 // Nombre de pions à aligner pour gagner par défaut
#define ROUGE 1 // Couleur de pion
#define JAUNE 2 // Couleur de pion
#define VIDE 0 // Absence de pion

// relatives à l'affichage
#define TCASES 50 // Taille des cases à l'affichage
#define CFONDGRILLE "dark blue" // Couleur de fond de l'écran
#define CROUGE "red" // Couleur des pions rouges
#define CJAUNE "yellow" // Couleur des pions jaunes
#define CVIDE "slate blue" // Couleur des cases vides de la grille
#define DISPLAY ":0" // Adresse du serveur X
#define HSTATUS 16 // Hauteur de la barre de status
#define POLICE "-adobe-helvetica-medium-r-normal--14-140-75-75-p-77-iso10646-1" // Police de caractères


/* Définition de type */
// Contient toutes les données relatives au jeu
typedef struct {
    int lgrille; // Largeur de la grille
    int hgrille; // Hauteur de la grille
    int **grille; // Grille de jeu contenant les pions
    int *historique; // Historique des coups joués
    int x; // Abcisse du dernier coup joué
    int y; // Ordonnée du dernier coup joué
    int tour; // Numéro du tour actuel ( commence à 0 )
    int joueur; // Joueur dont c'est le tour
    int puissance; // Nombre de pions à aligner pour gagner
    int tcases; // Taille des cases
    char *display; // Adresse du serveur X
    MlvType ecran;
} Donnees;


/* Prototypes */
// affichage.c
void affichage(Donnees *partie);
void affichage_pions(Donnees *partie);
void affichage_status(Donnees *partie);
void affichage_question(Donnees *partie, char *question);
void affichage_message(Donnees *partie, char *message);
void affichage_bouton(Donnees *partie, char *nom, char *couleur, int position);
void affiche_pion(Donnees *partie, int x, int y);
void aide(void);
void gagner(Donnees *partie);

// es.c
int entree(Donnees *partie);
void rejouer(Donnees *partie);
int quitter(Donnees *partie);
int reponse(Donnees *partie);
void options(Donnees *partie, int argc, char **argv);

// puissance_4.c
void init_partie(Donnees *partie);
void jeu(Donnees *partie);
void jouer(Donnees *partie, int colonne);
void annuler(Donnees *partie);
int puissance_4(Donnees *partie);
int recherche_pion(Donnees *partie, int colonne);
int ajout_pion(Donnees *partie, int colonne);
void enleve_pion(Donnees *partie);
void alloc_mem(Donnees *partie);
void libere_mem(Donnees *partie);
int nombre_pions(Donnees *partie, int dirx, int diry);
